"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const config_1 = require("./config");
const ids_json_1 = __importDefault(require("./ids.json"));
const web3_js_1 = require("@solana/web3.js");
const fs_1 = __importDefault(require("fs"));
const os_1 = __importDefault(require("os"));
const client_1 = require("./client");
const index_1 = require("./index");
const bn_js_1 = require("bn.js");
function mm() {
    return __awaiter(this, void 0, void 0, function* () {
        // load mango group and clients
        const config = new config_1.Config(ids_json_1.default);
        const groupName = process.env.GROUP || 'devnet.2';
        const mangoAccountName = process.env.MANGO_ACCOUNT_NAME;
        const groupIds = config.getGroupWithName(groupName);
        if (!groupIds) {
            throw new Error(`Group ${groupName} not found`);
        }
        const cluster = groupIds.cluster;
        const mangoProgramId = groupIds.mangoProgramId;
        const mangoGroupKey = groupIds.publicKey;
        const payer = new web3_js_1.Account(JSON.parse(fs_1.default.readFileSync(process.env.KEYPAIR || os_1.default.homedir() + '/.config/solana/id.json', 'utf-8')));
        console.log(`Payer: ${payer.publicKey.toBase58()}`);
        const connection = new web3_js_1.Connection(process.env.ENDPOINT_URL || config.cluster_urls[cluster], 'processed');
        const client = new client_1.MangoClient(connection, mangoProgramId);
        const mangoGroup = yield client.getMangoGroup(mangoGroupKey);
        // TODO make it fetch all mango accounts for owner and select highest value one
        const ownerAccounts = yield client.getMangoAccountsForOwner(mangoGroup, payer.publicKey, true);
        let mangoAccountPk;
        if (mangoAccountName) {
            for (const ownerAccount of ownerAccounts) {
                if (mangoAccountName === ownerAccount.name) {
                    mangoAccountPk = ownerAccount.publicKey;
                    break;
                }
            }
            if (!mangoAccountPk) {
                throw new Error('MANGO_ACCOUNT_NAME not found');
            }
        }
        else {
            const mangoAccountPkStr = process.env.MANGO_ACCOUNT_PUBKEY;
            if (!mangoAccountPkStr) {
                throw new Error('Please add env variable MANGO_ACCOUNT_PUBKEY or MANGO_ACCOUNT_NAME');
            }
            else {
                mangoAccountPk = new web3_js_1.PublicKey(mangoAccountPkStr);
            }
        }
        // TODO make it be able to quote all markets
        const marketName = process.env.MARKET;
        if (!marketName) {
            throw new Error('Please add env variable MARKET');
        }
        const perpMarketConfig = config_1.getPerpMarketByBaseSymbol(groupIds, marketName.toUpperCase());
        const marketIndex = perpMarketConfig.marketIndex;
        const perpMarket = yield client.getPerpMarket(perpMarketConfig.publicKey, perpMarketConfig.baseDecimals, perpMarketConfig.quoteDecimals);
        const sizePerc = parseFloat(process.env.SIZE_PERC || '0.1');
        const interval = parseInt(process.env.INTERVAL || '10000');
        const charge = parseFloat(process.env.CHARGE || '0.0010');
        const leanCoeff = parseFloat(process.env.LEAN_COEFF || '0.0005');
        const control = { isRunning: true, interval: interval };
        process.on('SIGINT', function () {
            console.log('Caught keyboard interrupt. Canceling orders');
            onExit(client, payer, mangoProgramId, mangoGroup, perpMarket, mangoAccountPk, control);
        });
        while (control.isRunning) {
            try {
                // get fresh data
                // get orderbooks, get perp markets, caches
                // TODO load pyth oracle for most accurate prices
                const [mangoCache, mangoAccount] = yield Promise.all([
                    // perpMarket.loadBids(connection),
                    // perpMarket.loadAsks(connection),
                    mangoGroup.loadCache(connection),
                    client.getMangoAccount(mangoAccountPk, mangoGroup.dexProgramId),
                ]);
                // TODO store the prices in an array to calculate volatility
                // Model logic
                const fairValue = mangoGroup.getPrice(marketIndex, mangoCache).toNumber();
                const equity = mangoAccount
                    .computeValue(mangoGroup, mangoCache)
                    .toNumber();
                const perpAccount = mangoAccount.perpAccounts[marketIndex];
                // TODO look at event queue as well for unprocessed fills
                const basePos = perpAccount.getBasePositionUi(perpMarket);
                // TODO volatility adjustment
                const size = (equity * sizePerc) / fairValue;
                const lean = (-leanCoeff * basePos) / size;
                const bidPrice = fairValue * (1 - charge + lean);
                const askPrice = fairValue * (1 + charge + lean);
                // TODO only requote if new prices significantly different from current
                const [nativeBidPrice, nativeBidSize] = perpMarket.uiToNativePriceQuantity(bidPrice, size);
                const [nativeAskPrice, nativeAskSize] = perpMarket.uiToNativePriceQuantity(askPrice, size);
                // cancel all, requote
                const cancelAllInstr = index_1.makeCancelAllPerpOrdersInstruction(mangoProgramId, mangoGroup.publicKey, mangoAccount.publicKey, payer.publicKey, perpMarket.publicKey, perpMarket.bids, perpMarket.asks, new bn_js_1.BN(20));
                const placeBidInstr = index_1.makePlacePerpOrderInstruction(mangoProgramId, mangoGroup.publicKey, mangoAccount.publicKey, payer.publicKey, mangoCache.publicKey, perpMarket.publicKey, perpMarket.bids, perpMarket.asks, perpMarket.eventQueue, mangoAccount.getOpenOrdersKeysInBasket(), nativeBidPrice, nativeBidSize, new bn_js_1.BN(Date.now()), 'buy', 'postOnlySlide');
                const placeAskInstruction = index_1.makePlacePerpOrderInstruction(mangoProgramId, mangoGroup.publicKey, mangoAccount.publicKey, payer.publicKey, mangoCache.publicKey, perpMarket.publicKey, perpMarket.bids, perpMarket.asks, perpMarket.eventQueue, mangoAccount.getOpenOrdersKeysInBasket(), nativeAskPrice, nativeAskSize, new bn_js_1.BN(Date.now()), 'sell', 'postOnlySlide');
                const tx = new web3_js_1.Transaction();
                tx.add(cancelAllInstr);
                tx.add(placeBidInstr);
                tx.add(placeAskInstruction);
                const txid = yield client.sendTransaction(tx, payer, []);
                console.log(`quoting successful: ${txid.toString()}`);
            }
            catch (e) {
                // TODO on keyboard interrupt cancel all and exit
                // sleep for some time and retry
                console.log(e);
            }
            finally {
                console.log(`sleeping for ${interval / 1000}s`);
                yield index_1.sleep(interval);
            }
        }
    });
}
function onExit(client, payer, mangoProgramId, mangoGroup, perpMarket, mangoAccountPk, control) {
    return __awaiter(this, void 0, void 0, function* () {
        yield index_1.sleep(control.interval);
        control.isRunning = false;
        const mangoAccount = yield client.getMangoAccount(mangoAccountPk, mangoGroup.dexProgramId);
        const cancelAllInstr = index_1.makeCancelAllPerpOrdersInstruction(mangoProgramId, mangoGroup.publicKey, mangoAccount.publicKey, payer.publicKey, perpMarket.publicKey, perpMarket.bids, perpMarket.asks, new bn_js_1.BN(20));
        const tx = new web3_js_1.Transaction();
        tx.add(cancelAllInstr);
        const txid = yield client.sendTransaction(tx, payer, []);
        console.log(`quoting successful: ${txid.toString()}`);
        process.exit();
    });
}
mm();
//# sourceMappingURL=mm.js.map